#include <EEPROM.h>
int button1;
int button2;
int past[1000];
int counter;
long sum;
int gain[4];
int lmVal; //val[4];
int loVal;
int hiVal;
int hmVal;
int talking = 0;
int dynamic = 0;
int justEntered = 0;
int timeSet = 0;
long timeStart = 0;
const int button1Pin = 34;
const int button2Pin = 35;

/* Function: Tune
 * Tunes the hearing aid
 */
void tune()
{
  digitalWrite(13,HIGH);
  setGain(22, 0);
  setGain(25, 0);
  setGain(28, 0);
  setGain(31, 0);
  tone(7,200);
  testGain(22);
  EEPROM.write(0, 0xAC);
  tone(7,700);
  testGain(25);
  tone(7,4000);
  testGain(28);
  tone(7,10000);
  testGain(31);
  noTone(7);
  for(int i=22; i<32; i=i+3)
  {
    setGain(i, EEPROM.read(i));
    gain[(i-22)/3] = EEPROM.read(i);
  }
  digitalWrite(13,LOW);
  while(digitalRead(button2Pin));
  delay(100);
}

/* Function: SetGain
 * Writes the given gain to Arduino output
 */
void setGain (int start, int gain) {
  for(int i = 0; i < 3; i++) {
    digitalWrite(start + i, (gain >> i) & 0x01);
  }  
}

/* Function: SaveGain
 * Saves the given gain to EEPROM
 */
void saveGain (int loc, int gain) {
    EEPROM.write(loc, gain);
}

/* Function: TestGain
 * Tests the gain of the given frequency
 */
void testGain (int start) {
  while(digitalRead(button2Pin));
  delay(100);
  button2 = 0;
  int gain = 1;
  setGain(start, gain);
  while(button2==0)
  {
    button1 = digitalRead(button1Pin);
    button2 = digitalRead(button2Pin);
    if(button1) {
        delay(500);
        if(gain==7) {
          gain=7;
          break;
        }
        else
          gain++;
        setGain(start, gain);
        while(digitalRead(button1Pin));
      }
  }
  setGain(start, 0);
  saveGain(start, gain);
  delay(100);
}

/* Function: Setup
 * Sets up everything
 */
void setup() {
 
  for(int i = 22; i < 34; i++)
    pinMode(i, OUTPUT);
  pinMode(34, INPUT);
  pinMode(35, INPUT);
  pinMode(7, OUTPUT);
  pinMode(13, OUTPUT);
  
  for(int i = 0; i <1000; i++)
    past[i] = 0;
  counter = 0;
  
  int check = EEPROM.read(0);
  if(check == 0xAC)
  {
    for(int i=22; i<32; i=i+3)
    {
      setGain(i, EEPROM.read(i));
      gain[(i-22)/3] = EEPROM.read(i);
    }
  }
  else
  {
    setGain(22,1);
    setGain(25,1);
    setGain(28,1);
    setGain(31,1); 
  }
 
  for(int i=0; i<1000; i++){
    delay(1);
    button1 = digitalRead(button1Pin);
    button2 = digitalRead(button2Pin);
    if(button1==1 || button2==1) {
      tune();
      break;
    }
  }
}

void loop() {
  if(digitalRead(button2Pin)) {
    dynamic = !dynamic;
    if(dynamic == 0)
      justEntered = 1;
    while(digitalRead(button2Pin));
    delay(100);
  }
  if(dynamic == 1)
  { 
    delay(1);
    lmVal = analogRead(A1);
    sum = sum - past[counter] + lmVal;
    past[counter] = lmVal;
  
    if(!talking && sum > 352000) //350(analogread) * 1000elements
    {
      talking = 1;
      analogWrite(13,128);
      if(gain[0]!=1)
        setGain(22, gain[0]-1);
      if(gain[2]!=1)
        setGain(28, gain[2]-1);
      if(gain[3]!=1)
        setGain(21, gain[3]-1);
      setGain(25, gain[1]+2);
    }
    if(talking && sum <350000)
    {
      talking = 0;
      timeStart = millis();
      timeSet = 1;
    }
    if(timeSet && (millis() - timeStart) > 5000) {
      timeSet = 0;
      if(sum > 352000) {
        talking = 1;
      } else {
        digitalWrite(13,LOW);
        setGain(22, gain[0]);
        setGain(25, gain[1]);
        setGain(28, gain[2]);
        setGain(31, gain[3]);
      }
    }
  
    //calculates next counter
    if(counter == 999)
      counter = 0;
    else
      counter++;
  }
  else
  {
    if(justEntered == 1)
    {
      digitalWrite(13,LOW);
      setGain(22, gain[0]);
      setGain(25, gain[1]);
      setGain(28, gain[2]);
      setGain(31, gain[3]);
      justEntered = 0;
    }
  }
}
